@@ -8,8 +8,6 @@
 # libradosstriper headers, boost headers, etc. - are already installed.
 #
 
-source $(dirname $0)/../ceph-helpers-root.sh
-
 trap cleanup EXIT
 
 SOURCES="hello_radosstriper.cc
@@ -22,8 +20,8 @@ hello_world_cpp
 "
 BINARIES="${BINARIES_TO_RUN}hello_radosstriper_cpp
 "
-DL_PREFIX="http://git.ceph.com/?p=ceph.git;a=blob_plain;hb=luminous;f=examples/librados/"
-#DL_PREFIX="https://raw.githubusercontent.com/ceph/ceph/luminous/examples/librados/"
+DL_PREFIX="http://git.ceph.com/?p=ceph.git;a=blob_plain;hb=mimic;f=examples/librados/"
+#DL_PREFIX="https://raw.githubusercontent.com/ceph/ceph/mimic/examples/librados/"
 DESTDIR=$(pwd)
 
 function cleanup () {
@@ -58,14 +56,6 @@ function run_binaries () {
 }
 
 pushd $DESTDIR
-case $(distro_id) in
-    centos|fedora|rhel|opensuse*|suse|sles)
-        install gcc-c++ make librados-devel;;
-    ubuntu|debian|devuan)
-        install g++ make librados-dev;;
-    *)
-        echo "$(distro_id) is unknown, $@ will have to be installed manually."
-esac
 get_sources
 check_sources
 make all-system
