@@ -23,9 +23,9 @@ struct AuthNoneAuthorizer : public AuthAuthorizer {
   AuthNoneAuthorizer() : AuthAuthorizer(CEPH_AUTH_NONE) { }
   bool build_authorizer(const EntityName &ename, uint64_t global_id) {
     __u8 struct_v = 1;
-    ::encode(struct_v, bl);
-    ::encode(ename, bl);
-    ::encode(global_id, bl);
+    encode(struct_v, bl);
+    encode(ename, bl);
+    encode(global_id, bl);
     return 0;
   }
   bool verify_reply(bufferlist::iterator& reply) override { return true; }
