@@ -26,7 +26,7 @@
 #include "global/signal_handler.h"
 #include "include/compat.h"
 #include "include/str_list.h"
-#include "common/admin_socket.h"
+#include "mon/MonClient.h"
 
 #include <pwd.h>
 #include <grp.h>
@@ -81,57 +81,89 @@ static int chown_path(const std::string &pathname, const uid_t owner, const gid_
   return r;
 }
 
-void global_pre_init(std::vector < const char * > *alt_def_args,
-		     std::vector < const char* >& args,
-		     uint32_t module_type, code_environment_t code_env,
-		     int flags)
+void global_pre_init(
+  const std::map<std::string,std::string> *defaults,
+  std::vector < const char* >& args,
+  uint32_t module_type, code_environment_t code_env,
+  int flags)
 {
   std::string conf_file_list;
   std::string cluster = "";
-  CephInitParameters iparams = ceph_argparse_early_args(args, module_type,
-							&cluster, &conf_file_list);
+
+  CephInitParameters iparams = ceph_argparse_early_args(
+    args, module_type,
+    &cluster, &conf_file_list);
+
   CephContext *cct = common_preinit(iparams, code_env, flags);
   cct->_conf->cluster = cluster;
   global_init_set_globals(cct);
   md_config_t *conf = cct->_conf;
 
-  if (alt_def_args)
-    conf->parse_argv(*alt_def_args);  // alternative default args
+  if (flags & (CINIT_FLAG_NO_DEFAULT_CONFIG_FILE|
+	       CINIT_FLAG_NO_MON_CONFIG)) {
+    conf->no_mon_config = true;
+  }
+
+  // alternate defaults
+  if (defaults) {
+    for (auto& i : *defaults) {
+      conf->set_val_default(i.first, i.second);
+    }
+  }
 
   int ret = conf->parse_config_files(c_str_or_null(conf_file_list),
 				     &cerr, flags);
   if (ret == -EDOM) {
-    dout_emergency("global_init: error parsing config file.\n");
+    cct->_log->flush();
+    cerr << "global_init: error parsing config file." << std::endl;
     _exit(1);
   }
   else if (ret == -ENOENT) {
     if (!(flags & CINIT_FLAG_NO_DEFAULT_CONFIG_FILE)) {
       if (conf_file_list.length()) {
-	ostringstream oss;
-	oss << "global_init: unable to open config file from search list "
-	    << conf_file_list << "\n";
-        dout_emergency(oss.str());
+	cct->_log->flush();
+	cerr << "global_init: unable to open config file from search list "
+	     << conf_file_list << std::endl;
         _exit(1);
       } else {
-        derr << "did not load config file, using default settings." << dendl;
+	cerr << "did not load config file, using default settings." << std::endl;
       }
     }
   }
   else if (ret) {
-    dout_emergency("global_init: error reading config file.\n");
+    cct->_log->flush();
+    cerr << "global_init: error reading config file." << std::endl;
     _exit(1);
   }
 
-  conf->parse_env(); // environment variables override
+  // environment variables override (CEPH_ARGS, CEPH_KEYRING)
+  conf->parse_env();
 
-  conf->parse_argv(args); // argv override
+  // command line (as passed by caller)
+  conf->parse_argv(args);
+
+  if (!conf->no_mon_config) {
+    // make sure our mini-session gets legacy values
+    conf->apply_changes(nullptr);
+
+    MonClient mc_bootstrap(g_ceph_context);
+    if (mc_bootstrap.get_monmap_and_config() < 0) {
+      cct->_log->flush();
+      cerr << "failed to fetch mon config (--no-mon-config to skip)"
+	   << std::endl;
+      _exit(1);
+    }
+  }
+
+  // do the --show-config[-val], if present in argv
+  conf->do_argv_commands();
 
   // Now we're ready to complain about config file parse errors
   g_conf->complain_about_parse_errors(g_ceph_context);
 }
 
 boost::intrusive_ptr<CephContext>
-global_init(std::vector < const char * > *alt_def_args,
+global_init(const std::map<std::string,std::string> *defaults,
 	    std::vector < const char* >& args,
 	    uint32_t module_type, code_environment_t code_env,
 	    int flags,
@@ -142,7 +174,7 @@ global_init(std::vector < const char * > *alt_def_args,
   if (run_pre_init) {
     // We will run pre_init from here (default).
     assert(!g_ceph_context && first_run);
-    global_pre_init(alt_def_args, args, module_type, code_env, flags);
+    global_pre_init(defaults, args, module_type, code_env, flags);
   } else {
     // Caller should have invoked pre_init manually.
     assert(g_ceph_context && first_run);
@@ -159,8 +191,10 @@ global_init(std::vector < const char * > *alt_def_args,
   int siglist[] = { SIGPIPE, 0 };
   block_signals(siglist, NULL);
 
-  if (g_conf->fatal_signal_handlers)
+  if (g_conf->fatal_signal_handlers) {
     install_standard_sighandlers();
+  }
+  register_assert_context(g_ceph_context);
 
   if (g_conf->log_flush_on_exit)
     g_ceph_context->_log->set_flush_on_exit();
@@ -184,30 +218,21 @@ global_init(std::vector < const char * > *alt_def_args,
     gid_t gid = 0;
     std::string uid_string;
     std::string gid_string;
-    std::string home_directory;
     if (g_conf->setuser.length()) {
-      char buf[4096];
-      struct passwd pa;
-      struct passwd *p = 0;
-
       uid = atoi(g_conf->setuser.c_str());
-      if (uid) {
-        getpwuid_r(uid, &pa, buf, sizeof(buf), &p);
-      } else {
+      if (!uid) {
+	char buf[4096];
+	struct passwd pa;
+	struct passwd *p = 0;
 	getpwnam_r(g_conf->setuser.c_str(), &pa, buf, sizeof(buf), &p);
-        if (!p) {
+	if (!p) {
 	  cerr << "unable to look up user '" << g_conf->setuser << "'"
 	       << std::endl;
 	  exit(1);
 	}
-
-        uid = p->pw_uid;
-        gid = p->pw_gid;
-        uid_string = g_conf->setuser;
-      }
-
-      if (p && p->pw_dir != nullptr) {
-        home_directory = std::string(p->pw_dir);
+	uid = p->pw_uid;
+	gid = p->pw_gid;
+	uid_string = g_conf->setuser;
       }
     }
     if (g_conf->setgroup.length() > 0) {
@@ -268,10 +293,6 @@ global_init(std::vector < const char * > *alt_def_args,
 	     << std::endl;
 	exit(1);
       }
-      if (setenv("HOME", home_directory.c_str(), 1) != 0) {
-	cerr << "warning: unable to set HOME to " << home_directory << ": "
-             << cpp_strerror(errno) << std::endl;
-      }
       priv_ss << "set uid:gid to " << uid << ":" << gid << " (" << uid_string << ":" << gid_string << ")";
     } else {
       priv_ss << "deferred set uid:gid to " << uid << ":" << gid << " (" << uid_string << ":" << gid_string << ")";
@@ -296,8 +317,6 @@ global_init(std::vector < const char * > *alt_def_args,
     }
   }
 
-  register_assert_context(g_ceph_context);
-
   // call all observers now.  this has the side-effect of configuring
   // and opening the log file immediately.
   g_conf->call_all_observers();
