@@ -28,7 +28,6 @@
 
 #include "Beacon.h"
 
-#include <math.h>
 #include <chrono>
 
 #define dout_context g_ceph_context
@@ -36,7 +35,9 @@
 #undef dout_prefix
 #define dout_prefix *_dout << "mds.beacon." << name << ' '
 
-Beacon::Beacon(CephContext *cct, MonClient *monc, boost::string_view name)
+using namespace std::chrono_literals;
+
+Beacon::Beacon(CephContext *cct, MonClient *monc, std::string_view name)
   :
     Dispatcher(cct),
     beacon_interval(g_conf->mds_beacon_interval),
@@ -56,14 +57,13 @@ void Beacon::shutdown()
   if (!finished) {
     finished = true;
     lock.unlock();
-    if (sender.joinable())
-      sender.join();
+    sender.join();
   }
 }
 
-void Beacon::init(const MDSMap* mdsmap)
+void Beacon::init(MDSMap const *mdsmap)
 {
-  std::unique_lock<std::mutex> lock(mutex);
+  std::unique_lock lock(mutex);
   assert(mdsmap != NULL);
 
   _notify_mdsmap(mdsmap);
@@ -87,7 +87,7 @@ void Beacon::init(const MDSMap* mdsmap)
         interval -= since;
       }
       dout(20) << "sender thread waiting interval " << interval << "s" << dendl;
-      c.wait_for(lock, interval*std::chrono::seconds(1));
+      c.wait_for(lock, interval*1s);
     }
   });
 }
@@ -125,7 +125,7 @@ bool Beacon::ms_dispatch(Message *m)
  */
 void Beacon::handle_mds_beacon(MMDSBeacon *m)
 {
-  std::unique_lock<std::mutex> lock(mutex);
+  std::unique_lock lock(mutex);
   assert(m != NULL);
 
   version_t seq = m->get_seq();
@@ -161,14 +161,14 @@ void Beacon::handle_mds_beacon(MMDSBeacon *m)
 
 void Beacon::send()
 {
-  std::unique_lock<std::mutex> lock(mutex);
+  std::unique_lock lock(mutex);
   _send();
 }
 
 
 void Beacon::send_and_wait(const double duration)
 {
-  std::unique_lock<std::mutex> lock(mutex);
+  std::unique_lock lock(mutex);
   _send();
   auto awaiting_seq = last_seq;
   dout(20) << __func__ << ": awaiting " << awaiting_seq
@@ -179,7 +179,7 @@ void Beacon::send_and_wait(const double duration)
     auto now = clock::now();
     auto s = duration*.95-std::chrono::duration<double>(now-start).count();
     if (s < 0) break;
-    cvar.wait_for(lock, s*std::chrono::seconds(1));
+    cvar.wait_for(lock, s*1s);
   }
 }
 
@@ -237,7 +237,7 @@ bool Beacon::_send()
  */
 void Beacon::notify_mdsmap(MDSMap const *mdsmap)
 {
-  std::unique_lock<std::mutex> lock(mutex);
+  std::unique_lock lock(mutex);
   assert(mdsmap != NULL);
 
   _notify_mdsmap(mdsmap);
@@ -258,14 +258,14 @@ void Beacon::_notify_mdsmap(MDSMap const *mdsmap)
 
 bool Beacon::is_laggy()
 {
-  std::unique_lock<std::mutex> lock(mutex);
+  std::unique_lock lock(mutex);
 
   auto now = clock::now();
   auto since = std::chrono::duration<double>(now-last_acked_stamp).count();
   if (since > g_conf->mds_beacon_grace) {
     if (!laggy) {
       dout(1) << "is_laggy " << since << " > " << g_conf->mds_beacon_grace
-	      << " since last acked beacon" << dendl;
+              << " since last acked beacon" << dendl;
     }
     laggy = true;
     auto last_reconnect = std::chrono::duration<double>(now-last_mon_reconnect).count();
@@ -281,9 +281,9 @@ bool Beacon::is_laggy()
   return false;
 }
 
-void Beacon::set_want_state(const MDSMap* mdsmap, MDSMap::DaemonState const newstate)
+void Beacon::set_want_state(MDSMap const *mdsmap, MDSMap::DaemonState const newstate)
 {
-  std::unique_lock<std::mutex> lock(mutex);
+  std::unique_lock lock(mutex);
 
   // Update mdsmap epoch atomically with updating want_state, so that when
   // we send a beacon with the new want state it has the latest epoch, and
@@ -308,7 +308,7 @@ void Beacon::set_want_state(const MDSMap* mdsmap, MDSMap::DaemonState const news
  */
 void Beacon::notify_health(MDSRank const *mds)
 {
-  std::unique_lock<std::mutex> lock(mutex);
+  std::unique_lock lock(mutex);
   if (!mds) {
     // No MDS rank held
     return;
@@ -390,31 +390,44 @@ void Beacon::notify_health(MDSRank const *mds)
     set<Session*> sessions;
     mds->sessionmap.get_client_session_set(sessions);
 
-    const auto recall_warning_threshold = g_conf->get_val<uint64_t>("mds_recall_warning_threshold");
-    const auto max_completed_requests = g_conf->mds_max_completed_requests;
-    const auto max_completed_flushes = g_conf->mds_max_completed_flushes;
+    auto mds_recall_state_timeout = g_conf->mds_recall_state_timeout;
+    auto last_recall = mds->mdcache->last_recall_state;
+    auto last_recall_span = std::chrono::duration<double>(clock::now()-last_recall).count();
+    bool recall_state_timedout = last_recall_span > mds_recall_state_timeout;
+
     std::list<MDSHealthMetric> late_recall_metrics;
     std::list<MDSHealthMetric> large_completed_requests_metrics;
     for (auto& session : sessions) {
-      const uint64_t recall_caps = fmax(0.0, session->get_recall_caps()); /* In Luminous: decay counter may go negative due to hit */
-      if (recall_caps > recall_warning_threshold) {
-        dout(2) << "Session " << *session <<
-             " is not releasing caps fast enough. Recalled caps at " << recall_caps
-          << " > " << recall_warning_threshold << " (mds_recall_warning_threshold)." << dendl;
-        std::ostringstream oss;
-        oss << "Client " << session->get_human_name() << " failing to respond to cache pressure";
-        MDSHealthMetric m(MDS_HEALTH_CLIENT_RECALL, HEALTH_WARN, oss.str());
-        m.metadata["client_id"] = stringify(session->get_client());
-        late_recall_metrics.push_back(m);
+      if (session->recalled_at != Session::time::min()) {
+        auto last_recall_sent = session->last_recall_sent;
+        auto recalled_at = session->recalled_at;
+        auto recalled_at_span = std::chrono::duration<double>(clock::now()-recalled_at).count();
+
+        dout(20) << "Session servicing RECALL " << session->info.inst
+          << ": " << recalled_at_span << "s ago " << session->recall_release_count
+          << "/" << session->recall_count << dendl;
+	if (recall_state_timedout || last_recall_sent < last_recall) {
+	  dout(20) << "  no longer recall" << dendl;
+	  session->clear_recalled_at();
+	} else if (recalled_at_span > mds_recall_state_timeout) {
+          dout(20) << "  exceeded timeout " << recalled_at_span << " vs. " << mds_recall_state_timeout << dendl;
+          std::ostringstream oss;
+	  oss << "Client " << session->get_human_name() << " failing to respond to cache pressure";
+          MDSHealthMetric m(MDS_HEALTH_CLIENT_RECALL, HEALTH_WARN, oss.str());
+          m.metadata["client_id"] = stringify(session->get_client());
+          late_recall_metrics.push_back(m);
+        } else {
+          dout(20) << "  within timeout " << recalled_at_span << " vs. " << mds_recall_state_timeout << dendl;
+        }
       }
       if ((session->get_num_trim_requests_warnings() > 0 &&
-	   session->get_num_completed_requests() >= max_completed_requests) ||
+	   session->get_num_completed_requests() >= g_conf->mds_max_completed_requests) ||
 	  (session->get_num_trim_flushes_warnings() > 0 &&
-	   session->get_num_completed_flushes() >= max_completed_flushes)) {
+	   session->get_num_completed_flushes() >= g_conf->mds_max_completed_flushes)) {
 	std::ostringstream oss;
 	oss << "Client " << session->get_human_name() << " failing to advance its oldest client/flush tid. ";
 	MDSHealthMetric m(MDS_HEALTH_CLIENT_OLDEST_TID, HEALTH_WARN, oss.str());
-	m.metadata["client_id"] = stringify(session->info.inst.name.num());
+	m.metadata["client_id"] = stringify(session->get_client());
 	large_completed_requests_metrics.push_back(m);
       }
     }
@@ -450,7 +463,7 @@ void Beacon::notify_health(MDSRank const *mds)
     if (slow) {
       dout(20) << slow << " slow request found" << dendl;
       std::ostringstream oss;
-      oss << slow << " slow requests are blocked > " << g_conf->mds_op_complaint_time << " sec";
+      oss << slow << " slow requests are blocked > " << g_conf->mds_op_complaint_time << " secs";
 
       MDSHealthMetric m(MDS_HEALTH_SLOW_REQUEST, HEALTH_WARN, oss.str());
       health.metrics.push_back(m);
@@ -499,7 +512,7 @@ void Beacon::notify_health(MDSRank const *mds)
 
 MDSMap::DaemonState Beacon::get_want_state() const
 {
-  std::unique_lock<std::mutex> lock(mutex);
+  std::unique_lock lock(mutex);
   return want_state;
 }
 
