@@ -95,6 +95,14 @@ DaemonStatePtr DaemonStateIndex::get(const DaemonKey &key)
   }
 }
 
+void DaemonStateIndex::rm(const DaemonKey &key)
+{
+  RWLock::WLocker l(lock);
+  if (all.count(key)) {
+    _erase(key);
+  }
+}
+
 void DaemonStateIndex::cull(const std::string& svc_name,
 			    const std::set<std::string>& names_exist)
 {
@@ -157,10 +165,10 @@ void DaemonPerfCounters::update(MMgrReport *report)
     uint64_t avgcount = 0;
     uint64_t avgcount2 = 0;
 
-    ::decode(val, p);
+    decode(val, p);
     if (t.type & PERFCOUNTER_LONGRUNAVG) {
-      ::decode(avgcount, p);
-      ::decode(avgcount2, p);
+      decode(avgcount, p);
+      decode(avgcount2, p);
       instances_it->second.push_avg(now, val, avgcount);
     } else {
       instances_it->second.push(now, val);
