@@ -165,9 +165,9 @@ void handle_connection(RGWProcessEnv& env, Stream& stream,
     RGWRequest req{env.store->get_new_req_id()};
 
     auto& socket = stream.lowest_layer();
-    StreamIO<Stream> real_client{stream, parser, buffer, is_ssl,
-                                 socket.local_endpoint(),
-                                 socket.remote_endpoint()};
+    StreamIO real_client{stream, parser, buffer, is_ssl,
+                         socket.local_endpoint(),
+                         socket.remote_endpoint()};
 
     auto real_client_io = rgw::io::add_reordering(
                             rgw::io::add_buffering(cct,
@@ -256,7 +256,8 @@ unsigned short parse_port(const char *input, boost::system::error_code& ec)
   }
   return port;
 }
-tcp::endpoint parse_endpoint(BOOST_ASIO_STRING_VIEW_PARAM input,
+	
+tcp::endpoint parse_endpoint(boost::asio::string_view input,
                              unsigned short default_port,
                              boost::system::error_code& ec)
 {
@@ -283,6 +284,8 @@ tcp::endpoint parse_endpoint(BOOST_ASIO_STRING_VIEW_PARAM input,
         auto port_str = input.substr(addr_end + 2);
         endpoint.port(parse_port(port_str.data(), ec));
       }
+    } else {
+      endpoint.port(default_port);
     }
     auto addr = input.substr(addr_begin, addr_end - addr_begin);
     endpoint.address(boost::asio::ip::make_address_v6(addr, ec));
@@ -294,6 +297,8 @@ tcp::endpoint parse_endpoint(BOOST_ASIO_STRING_VIEW_PARAM input,
       if (ec) {
         return endpoint;
       }
+    } else {
+      endpoint.port(default_port);
     }
     auto addr = input.substr(0, colon);
     endpoint.address(boost::asio::ip::make_address_v4(addr, ec));
@@ -480,8 +485,7 @@ void AsioFrontend::accept(Listener& l, boost::system::error_code ec)
   } else if (ec == boost::asio::error::operation_aborted) {
     return;
   } else if (ec) {
-    ldout(ctx(), 1) << "accept failed: " << ec.message() << dendl;
-    return;
+    throw ec;
   }
   auto socket = std::move(l.socket);
   tcp::no_delay options(l.use_nodelay);
@@ -494,8 +498,8 @@ void AsioFrontend::accept(Listener& l, boost::system::error_code ec)
   // spawn a coroutine to handle the connection
 #ifdef WITH_RADOSGW_BEAST_OPENSSL
   if (l.use_ssl) {
-    boost::asio::spawn(service, std::bind(
-      [this] (boost::asio::yield_context yield, tcp::socket& s) mutable {
+    boost::asio::spawn(service,
+      [this, s=std::move(socket)] (boost::asio::yield_context yield) mutable {
         // wrap the socket in an ssl stream
         ssl::stream<tcp::socket&> stream{s, *ssl_context};
         beast::flat_buffer buffer;
@@ -514,18 +518,18 @@ void AsioFrontend::accept(Listener& l, boost::system::error_code ec)
           stream.async_shutdown(yield[ec]);
         }
         s.shutdown(tcp::socket::shutdown_both, ec);
-      }, std::placeholders::_1, std::move(socket)));
+      });
   } else {
 #else
   {
 #endif // WITH_RADOSGW_BEAST_OPENSSL
-    boost::asio::spawn(service, std::bind(
-      [this] (boost::asio::yield_context yield, tcp::socket& s) mutable {
+    boost::asio::spawn(service,
+      [this, s=std::move(socket)] (boost::asio::yield_context yield) mutable {
         beast::flat_buffer buffer;
         boost::system::error_code ec;
         handle_connection(env, s, buffer, false, ec, yield);
         s.shutdown(tcp::socket::shutdown_both, ec);
-      }, std::placeholders::_1, std::move(socket)));
+      });
   }
 }
 
