@@ -41,7 +41,6 @@
 #define dout_context g_ceph_context
 #define dout_subsys ceph_subsys_crush
 
-using namespace std;
 
 const char *infn = "stdin";
 
@@ -254,6 +253,42 @@ struct layer_t {
   int size;
 };
 
+template<typename... Args>
+bool argparse_withargs(std::vector<const char*> &args,
+		       std::vector<const char*>::iterator& i,
+		       std::ostream& oss,
+		       const char* opt,
+		       Args*... opts)
+{
+  if (!ceph_argparse_flag(args, i, opt, nullptr)) {
+    return false;
+  }
+  auto parse = [&](auto& opt) {
+    if (i == args.end()) {
+      oss << "expecting additional argument to " << opt;
+      return false;
+    }
+    using opt_t = std::remove_pointer_t<decay_t<decltype(opt)>>;
+    string err;
+    if constexpr (std::is_same_v<opt_t, string>) {
+      opt->assign(*i);
+    } else if constexpr (is_same_v<opt_t, int>) {
+      *opt = strict_strtol(*i, 10, &err);
+    } else if constexpr (is_same_v<opt_t, float>) {
+      *opt = strict_strtof(*i, &err);
+    }
+    i = args.erase(i);
+    if (err.empty())
+      return true;
+    else {
+      oss << err;
+      return false;
+    }
+  };
+  (... && parse(opts));
+  return true;
+}
+
 int do_add_bucket(CephContext* cct,
 		  const char* me,
 		  CrushWrapper& crush,
@@ -270,21 +305,18 @@ int do_add_bucket(CephContext* cct,
     cerr << me << " bad bucket type: " << add_type << std::endl;
     return -EINVAL;
   }
-  int r = 0;
-  r = crush.add_bucket(0, 0, CRUSH_HASH_DEFAULT, type, 0, nullptr, nullptr, &bucketno);
-  if (r < 0) {
+  if (int r = crush.add_bucket(0, 0, CRUSH_HASH_DEFAULT, type, 0, nullptr, nullptr, &bucketno);
+      r < 0) {
     cerr << me << " unable to add bucket: " << cpp_strerror(r) << std::endl;
     return r;
   }
-  r = crush.set_item_name(bucketno, add_name);
-  if (r < 0) {
+  if (int r = crush.set_item_name(bucketno, add_name); r < 0) {
     cerr << me << " bad bucket name: " << add_name << std::endl;
     return r;
   }
   if (!add_loc.empty()) {
     if (!crush.check_item_loc(cct, bucketno, add_loc, (int*)nullptr)) {
-      r = crush.move_bucket(cct, bucketno, add_loc);
-      if (r < 0) {
+      if (int r = crush.move_bucket(cct, bucketno, add_loc); r < 0) {
 	cerr << me << " error moving bucket '" << add_name << "' to " << add_loc << std::endl;
 	return r;
       }
@@ -332,6 +364,14 @@ int main(int argc, const char **argv)
 {
   vector<const char*> args;
   argv_to_vec(argc, argv, args);
+  if (args.empty()) {
+    cerr << argv[0] << ": -h or --help for usage" << std::endl;
+    exit(1);
+  }
+  if (ceph_argparse_need_usage(args)) {
+    usage();
+    exit(0);
+  }
 
   const char *me = argv[0];
   std::string infn, srcfn, outfn, add_name, add_type, remove_name, reweight_name;
@@ -392,9 +432,8 @@ int main(int argc, const char **argv)
 
   // we use -c, don't confuse the generic arg parsing
   // only parse arguments from CEPH_ARGS, if in the environment
-  vector<const char *> env_args;
-  env_to_vec(env_args);
-  auto cct = global_init(NULL, env_args, CEPH_ENTITY_TYPE_CLIENT,
+  vector<const char *> empty_args;
+  auto cct = global_init(NULL, empty_args, CEPH_ENTITY_TYPE_CLIENT,
 			 CODE_ENVIRONMENT_UTILITY,
 			 CINIT_FLAG_NO_DEFAULT_CONFIG_FILE);
   // crushtool times out occasionally when quits. so do not
@@ -412,9 +451,6 @@ int main(int argc, const char **argv)
   for (std::vector<const char*>::iterator i = args.begin(); i != args.end(); ) {
     if (ceph_argparse_double_dash(args, i)) {
       break;
-    } else if (ceph_argparse_flag(args, i, "-h", "--help", (char*)NULL)) {
-      usage();
-      return EXIT_SUCCESS;
     } else if (ceph_argparse_witharg(args, i, &val, "-d", "--decompile", (char*)NULL)) {
       infn = val;
       decompile = true;
@@ -557,21 +593,15 @@ int main(int argc, const char **argv)
       }
       add_name.assign(*i);
       i = args.erase(i);
-    } else if (ceph_argparse_witharg(args, i, &val, err, "--add-bucket", (char*)NULL)) {
-      add_name.assign(val);
+    } else if (argparse_withargs(args, i, err, "--add-bucket",
+				 &add_name, &add_type)) {
       if (!err.str().empty()) {
 	cerr << err.str() << std::endl;
 	return EXIT_FAILURE;
       }
-      if (i == args.end()) {
-	cerr << "expecting additional argument to --add-bucket" << std::endl;
-	return EXIT_FAILURE;
-      }
-      add_type = *i;
-      i = args.erase(i);
       add_bucket = true;
-    } else if (ceph_argparse_witharg(args, i, &val, err, "--move", (char*)NULL)) {
-      move_name.assign(val);
+    } else if (argparse_withargs(args, i, err, "--move",
+				 &move_name)) {
       if (!err.str().empty()) {
 	cerr << err.str() << std::endl;
 	return EXIT_FAILURE;
@@ -1092,8 +1122,7 @@ int main(int argc, const char **argv)
   }
 
   if (add_bucket) {
-    int r = do_add_bucket(cct.get(), me, crush, add_name, add_type, add_loc);
-    if (!r) {
+    if (int r = do_add_bucket(cct.get(), me, crush, add_name, add_type, add_loc); !r) {
       modified = true;
     } else {
       return r;
@@ -1101,8 +1130,7 @@ int main(int argc, const char **argv)
   }
 
   if (move_item) {
-    int r = do_move_item(cct.get(), me, crush, move_name, add_loc);
-    if (!r) {
+    if (int r = do_move_item(cct.get(), me, crush, move_name, add_loc); !r) {
       modified = true;
     } else {
       return r;
