@@ -19,13 +19,10 @@
 #include "common/errno.h"
 #include "common/safe_io.h"
 #include "mon/health_check.h"
-#include <time.h>
-#include <algorithm>
 
 #include "global/global_init.h"
 #include "osd/OSDMap.h"
 
-using namespace std;
 
 void usage()
 {
@@ -48,12 +45,11 @@ void usage()
   cout << "                           commands to <file> [default: - for stdout]" << std::endl;
   cout << "   --upmap <file>          calculate pg upmap entries to balance pg layout" << std::endl;
   cout << "                           writing commands to <file> [default: - for stdout]" << std::endl;
-  cout << "   --upmap-max <max-count> set max upmap entries to calculate [default: 10]" << std::endl;
+  cout << "   --upmap-max <max-count> set max upmap entries to calculate [default: 100]" << std::endl;
   cout << "   --upmap-deviation <max-deviation>" << std::endl;
-  cout << "                           max deviation from target [default: 5]" << std::endl;
+  cout << "                           max deviation from target [default: .01]" << std::endl;
   cout << "   --upmap-pool <poolname> restrict upmap balancing to 1 or more pools" << std::endl;
   cout << "   --upmap-save            write modified OSDMap with upmap changes" << std::endl;
-  cout << "   --upmap-active          Act like an active balancer, keep applying changes until balanced" << std::endl;
   exit(1);
 }
 
@@ -92,7 +88,14 @@ int main(int argc, const char **argv)
 {
   vector<const char*> args;
   argv_to_vec(argc, argv, args);
-  env_to_vec(args);
+  if (args.empty()) {
+    cerr << argv[0] << ": -h or --help for usage" << std::endl;
+    exit(1);
+  }
+  if (ceph_argparse_need_usage(args)) {
+    usage();
+    exit(0);
+  }
 
   auto cct = global_init(NULL, args, CEPH_ENTITY_TYPE_CLIENT,
 			 CODE_ENVIRONMENT_UTILITY,
@@ -110,8 +113,8 @@ int main(int argc, const char **argv)
   bool createpool = false;
   bool create_from_conf = false;
   int num_osd = 0;
-  int pg_bits = g_conf->osd_pg_bits;
-  int pgp_bits = g_conf->osd_pgp_bits;
+  int pg_bits = 6;
+  int pgp_bits = 6;
   bool clobber = false;
   bool modified = false;
   std::string export_crush, import_crush, test_map_pg, test_map_object;
@@ -130,9 +133,8 @@ int main(int argc, const char **argv)
   bool upmap_save = false;
   bool health = false;
   std::string upmap_file = "-";
-  int upmap_max = 10;
-  int upmap_deviation = 5;
-  bool upmap_active = false;
+  int upmap_max = 100;
+  float upmap_deviation = .01;
   std::set<std::string> upmap_pools;
   int64_t pg_num = -1;
   bool test_map_pgs_dump_all = false;
@@ -142,8 +144,6 @@ int main(int argc, const char **argv)
   for (std::vector<const char*>::iterator i = args.begin(); i != args.end(); ) {
     if (ceph_argparse_double_dash(args, i)) {
       break;
-    } else if (ceph_argparse_flag(args, i, "-h", "--help", (char*)NULL)) {
-      usage();
     } else if (ceph_argparse_flag(args, i, "-p", "--print", (char*)NULL)) {
       print = true;
     } else if (ceph_argparse_witharg(args, i, &val, err, "--dump", (char*)NULL)) {
@@ -156,6 +156,8 @@ int main(int argc, const char **argv)
       if (!val.empty() && val != "plain") {
 	tree_formatter.reset(Formatter::create(val, "", "json"));
       }
+    } else if (ceph_argparse_witharg(args, i, &pg_bits, err, "--osd-pg-bits", (char*)NULL)) {
+    } else if (ceph_argparse_witharg(args, i, &pgp_bits, err, "--osd-pgp-bits", (char*)NULL)) {
     } else if (ceph_argparse_witharg(args, i, &upmap_file, "--upmap-cleanup", (char*)NULL)) {
       upmap_cleanup = true;
     } else if (ceph_argparse_witharg(args, i, &upmap_file, "--upmap-save", (char*)NULL)) {
@@ -173,8 +175,6 @@ int main(int argc, const char **argv)
 	exit(EXIT_FAILURE);
       }
       createsimple = true;
-    } else if (ceph_argparse_flag(args, i, "--upmap-active", (char*)NULL)) {
-      upmap_active = true;
     } else if (ceph_argparse_flag(args, i, "--health", (char*)NULL)) {
       health = true;
     } else if (ceph_argparse_flag(args, i, "--with-default-pool", (char*)NULL)) {
@@ -243,10 +243,6 @@ int main(int argc, const char **argv)
     cerr << me << ": too many arguments" << std::endl;
     usage();
   }
-  if (upmap_deviation < 1) {
-    cerr << me << ": upmap-deviation must be >= 1" << std::endl;
-    usage();
-  }
   fn = args[0];
 
   if (range_first >= 0 && range_last >= 0) {
@@ -312,7 +308,6 @@ int main(int argc, const char **argv)
       num_osd = -1;
     }
     uuid_d fsid;
-    memset(&fsid, 0, sizeof(uuid_d));
     if (createpool) {
       osdmap.build_simple_with_pool(
 	g_ceph_context, 0, fsid, num_osd, pg_bits, pgp_bits);
@@ -347,7 +342,7 @@ int main(int argc, const char **argv)
   int upmap_fd = STDOUT_FILENO;
   if (upmap || upmap_cleanup) {
     if (upmap_file != "-") {
-      upmap_fd = ::open(upmap_file.c_str(), O_CREAT|O_WRONLY|O_TRUNC, 0644);
+      upmap_fd = ::open(upmap_file.c_str(), O_CREAT|O_WRONLY, 0644);
       if (upmap_fd < 0) {
 	cerr << "error opening " << upmap_file << ": " << cpp_strerror(errno)
 	     << std::endl;
@@ -371,105 +366,35 @@ int main(int argc, const char **argv)
     cout << "upmap, max-count " << upmap_max
 	 << ", max deviation " << upmap_deviation
 	 << std::endl;
-    vector<int64_t> pools;
-    set<int64_t> upmap_pool_nums;
+    OSDMap::Incremental pending_inc(osdmap.get_epoch()+1);
+    pending_inc.fsid = osdmap.get_fsid();
+    set<int64_t> pools;
     for (auto& s : upmap_pools) {
       int64_t p = osdmap.lookup_pg_pool_name(s);
       if (p < 0) {
-	cerr << " pool " << s << " does not exist" << std::endl;
+	cerr << " pool '" << s << "' does not exist" << std::endl;
 	exit(1);
       }
-      pools.push_back(p);
-      upmap_pool_nums.insert(p);
+      pools.insert(p);
     }
-    if (!pools.empty()) {
+    if (!pools.empty())
       cout << " limiting to pools " << upmap_pools << " (" << pools << ")"
 	   << std::endl;
+    int changed = osdmap.calc_pg_upmaps(
+      g_ceph_context, upmap_deviation,
+      upmap_max, pools,
+      &pending_inc);
+    if (changed) {
+      print_inc_upmaps(pending_inc, upmap_fd);
+      if (upmap_save) {
+	int r = osdmap.apply_incremental(pending_inc);
+	assert(r == 0);
+	modified = true;
+      }
     } else {
-      mempool::osdmap::map<int64_t,pg_pool_t> opools = osdmap.get_pools();
-      for (auto& i : opools) {
-         pools.push_back(i.first);
-      }
+      cout << "no upmaps proposed" << std::endl;
     }
-    if (pools.empty()) {
-      cout << "No pools available" << std::endl;
-      goto skip_upmap;
-    }
-    int rounds = 0;
-    struct timespec round_start;
-    int r = clock_gettime(CLOCK_MONOTONIC, &round_start);
-    assert(r == 0);
-    do {
-      std::random_device rd;
-      std::shuffle(pools.begin(), pools.end(), std::mt19937{rd()});
-      cout << "pools ";
-      for (auto& i: pools)
-        cout << osdmap.get_pool_name(i) << " ";
-      cout << std::endl;
-      OSDMap::Incremental pending_inc(osdmap.get_epoch()+1);
-      pending_inc.fsid = osdmap.get_fsid();
-      int total_did = 0;
-      int left = upmap_max;
-      struct timespec begin, end;
-      r = clock_gettime(CLOCK_MONOTONIC, &begin);
-      assert(r == 0);
-      for (auto& i: pools) {
-        set<int64_t> one_pool;
-        one_pool.insert(i);
-        int did = osdmap.calc_pg_upmaps(
-          g_ceph_context, upmap_deviation,
-          left, one_pool,
-          &pending_inc);
-        total_did += did;
-        left -= did;
-        if (left <= 0)
-          break;
-      }
-      r = clock_gettime(CLOCK_MONOTONIC, &end);
-      assert(r == 0);
-      cout << "prepared " << total_did << "/" << upmap_max  << " changes" << std::endl;
-      float elapsed_time = (end.tv_sec - begin.tv_sec) + 1.0e-9*(end.tv_nsec - begin.tv_nsec);
-      if (upmap_active)
-        cout << "Time elapsed " << elapsed_time << " secs" << std::endl;
-      if (total_did > 0) {
-        print_inc_upmaps(pending_inc, upmap_fd);
-        if (upmap_save || upmap_active) {
-	  int r = osdmap.apply_incremental(pending_inc);
-	  assert(r == 0);
-	  if (upmap_save)
-	    modified = true;
-        }
-      } else {
-        cout << "Unable to find further optimization, "
-	     << "or distribution is already perfect"
-	     << std::endl;
-        if (upmap_active) {
-          map<int,set<pg_t>> pgs_by_osd;
-          for (auto& i : osdmap.get_pools()) {
-            if (!upmap_pool_nums.empty() && !upmap_pool_nums.count(i.first))
-              continue;
-            for (unsigned ps = 0; ps < i.second.get_pg_num(); ++ps) {
-              pg_t pg(ps, i.first);
-              vector<int> up;
-              osdmap.pg_to_up_acting_osds(pg, &up, nullptr, nullptr, nullptr);
-              //ldout(cct, 20) << __func__ << " " << pg << " up " << up << dendl;
-              for (auto osd : up) {
-                if (osd != CRUSH_ITEM_NONE)
-                  pgs_by_osd[osd].insert(pg);
-              }
-            }
-          }
-          for (auto& i : pgs_by_osd)
-            cout << "osd." << i.first << " pgs " << i.second.size() << std::endl;
-          float elapsed_time = (end.tv_sec - round_start.tv_sec) + 1.0e-9*(end.tv_nsec - round_start.tv_nsec);
-          cout << "Total time elapsed " << elapsed_time << " secs, " << rounds << " rounds" << std::endl;
-        }
-        break;
-      }
-      ++rounds;
-    } while(upmap_active);
   }
-skip_upmap:
   if (upmap_file != "-") {
     ::close(upmap_fd);
   }
@@ -686,7 +611,7 @@ skip_upmap:
 	   ++p) {
 	const pg_pool_t *pool = osdmap.get_pg_pool(p->first);
 	for (ps_t ps = 0; ps < pool->get_pg_num(); ps++) {
-	  pg_t pgid(ps, p->first, -1);
+	  pg_t pgid(ps, p->first);
 	  for (int i=0; i<100; i++) {
 	    cout << pgid << " attempt " << i << std::endl;
 
