@@ -3,6 +3,7 @@
 
 #include "include/compat.h"
 #include "common/Formatter.h"
+#include "common/admin_socket.h"
 #include "common/debug.h"
 #include "common/errno.h"
 #include "include/stringify.h"
@@ -194,8 +195,8 @@ public:
     commands.clear();
   }
 
-  bool call(std::string command, cmdmap_t& cmdmap, std::string format,
-	    bufferlist& out) override {
+  bool call(std::string_view command, const cmdmap_t& cmdmap,
+	    std::string_view format, bufferlist& out) override {
     auto i = commands.find(command);
     assert(i != commands.end());
     Formatter *f = Formatter::create(format);
@@ -207,7 +208,8 @@ public:
   }
 
 private:
-  typedef std::map<std::string, ImageReplayerAdminSocketCommand<I> *> Commands;
+  typedef std::map<std::string, ImageReplayerAdminSocketCommand<I>*,
+		   std::less<>> Commands;
 
   AdminSocket *admin_socket;
   Commands commands;
@@ -252,14 +254,12 @@ void ImageReplayer<I>::RemoteJournalerListener::handle_update(
 
 template <typename I>
 ImageReplayer<I>::ImageReplayer(Threads<I> *threads,
-                                ImageDeleter<I>* image_deleter,
                                 InstanceWatcher<I> *instance_watcher,
                                 RadosRef local,
                                 const std::string &local_mirror_uuid,
                                 int64_t local_pool_id,
                                 const std::string &global_image_id) :
   m_threads(threads),
-  m_image_deleter(image_deleter),
   m_instance_watcher(instance_watcher),
   m_local(local),
   m_local_mirror_uuid(local_mirror_uuid),
@@ -332,7 +332,7 @@ void ImageReplayer<I>::add_peer(const std::string &peer_uuid,
 
 template <typename I>
 void ImageReplayer<I>::set_state_description(int r, const std::string &desc) {
-  dout(10) << r << " " << desc << dendl;
+  dout(20) << r << " " << desc << dendl;
 
   Mutex::Locker l(m_lock);
   m_last_r = r;
@@ -342,7 +342,7 @@ void ImageReplayer<I>::set_state_description(int r, const std::string &desc) {
 template <typename I>
 void ImageReplayer<I>::start(Context *on_finish, bool manual)
 {
-  dout(10) << "on_finish=" << on_finish << dendl;
+  dout(20) << "on_finish=" << on_finish << dendl;
 
   int r = 0;
   {
@@ -376,61 +376,39 @@ void ImageReplayer<I>::start(Context *on_finish, bool manual)
     return;
   }
 
-  r = m_local->ioctx_create2(m_local_pool_id, m_local_ioctx);
+  m_local_ioctx.reset(new librados::IoCtx{});
+  r = m_local->ioctx_create2(m_local_pool_id, *m_local_ioctx);
   if (r < 0) {
+    m_local_ioctx.reset();
+
     derr << "error opening ioctx for local pool " << m_local_pool_id
          << ": " << cpp_strerror(r) << dendl;
     on_start_fail(r, "error opening local pool");
     return;
   }
 
-  wait_for_deletion();
-}
-
-template <typename I>
-void ImageReplayer<I>::wait_for_deletion() {
-  dout(20) << dendl;
-
-  Context *ctx = create_context_callback<
-    ImageReplayer, &ImageReplayer<I>::handle_wait_for_deletion>(this);
-  m_image_deleter->wait_for_scheduled_deletion(
-    m_local_pool_id, m_global_image_id, ctx, false);
-}
-
-template <typename I>
-void ImageReplayer<I>::handle_wait_for_deletion(int r) {
-  dout(20) << "r=" << r << dendl;
-
-  if (r == -ECANCELED) {
-    on_start_fail(0, "");
-    return;
-  } else if (r < 0) {
-    on_start_fail(r, "error waiting for image deletion");
-    return;
-  }
-
   prepare_local_image();
 }
 
 template <typename I>
 void ImageReplayer<I>::prepare_local_image() {
-  dout(20) << dendl;
+  dout(10) << dendl;
 
   m_local_image_id = "";
   Context *ctx = create_context_callback<
     ImageReplayer, &ImageReplayer<I>::handle_prepare_local_image>(this);
   auto req = PrepareLocalImageRequest<I>::create(
-    m_local_ioctx, m_global_image_id, &m_local_image_id, &m_local_image_name,
+    *m_local_ioctx, m_global_image_id, &m_local_image_id, &m_local_image_name,
     &m_local_image_tag_owner, m_threads->work_queue, ctx);
   req->send();
 }
 
 template <typename I>
 void ImageReplayer<I>::handle_prepare_local_image(int r) {
-  dout(20) << "r=" << r << dendl;
+  dout(10) << "r=" << r << dendl;
 
   if (r == -ENOENT) {
-    dout(20) << "local image does not exist" << dendl;
+    dout(10) << "local image does not exist" << dendl;
   } else if (r < 0) {
     on_start_fail(r, "error preparing local image for replay");
     return;
@@ -444,7 +422,7 @@ void ImageReplayer<I>::handle_prepare_local_image(int r) {
 
 template <typename I>
 void ImageReplayer<I>::prepare_remote_image() {
-  dout(20) << dendl;
+  dout(10) << dendl;
   if (m_peers.empty()) {
     // technically nothing to bootstrap, but it handles the status update
     bootstrap();
@@ -466,17 +444,17 @@ void ImageReplayer<I>::prepare_remote_image() {
 
 template <typename I>
 void ImageReplayer<I>::handle_prepare_remote_image(int r) {
-  dout(20) << "r=" << r << dendl;
+  dout(10) << "r=" << r << dendl;
 
   assert(r < 0 ? m_remote_journaler == nullptr : m_remote_journaler != nullptr);
   if (r < 0 && !m_local_image_id.empty() &&
       m_local_image_tag_owner == librbd::Journal<>::LOCAL_MIRROR_UUID) {
     // local image is primary -- fall-through
   } else if (r == -ENOENT) {
-    dout(20) << "remote image does not exist" << dendl;
+    dout(10) << "remote image does not exist" << dendl;
 
     // TODO need to support multiple remote images
-    if (!m_local_image_id.empty() &&
+    if (m_remote_image.image_id.empty() && !m_local_image_id.empty() &&
         m_local_image_tag_owner == m_remote_image.mirror_uuid) {
       // local image exists and is non-primary and linked to the missing
       // remote image
@@ -497,7 +475,7 @@ void ImageReplayer<I>::handle_prepare_remote_image(int r) {
 
 template <typename I>
 void ImageReplayer<I>::bootstrap() {
-  dout(20) << dendl;
+  dout(10) << dendl;
 
   if (!m_local_image_id.empty() &&
       m_local_image_tag_owner == librbd::Journal<>::LOCAL_MIRROR_UUID) {
@@ -510,22 +488,19 @@ void ImageReplayer<I>::bootstrap() {
     return;
   }
 
-  BootstrapRequest<I> *request = nullptr;
+  Context *ctx = create_context_callback<
+    ImageReplayer, &ImageReplayer<I>::handle_bootstrap>(this);
+
+  BootstrapRequest<I> *request = BootstrapRequest<I>::create(
+    *m_local_ioctx, m_remote_image.io_ctx, m_instance_watcher,
+    &m_local_image_ctx, m_local_image_id, m_remote_image.image_id,
+    m_global_image_id, m_threads->work_queue, m_threads->timer,
+    &m_threads->timer_lock, m_local_mirror_uuid, m_remote_image.mirror_uuid,
+    m_remote_journaler, &m_client_state, &m_client_meta, ctx,
+    &m_resync_requested, &m_progress_cxt);
+
   {
     Mutex::Locker locker(m_lock);
-    if (on_start_interrupted(m_lock)) {
-      return;
-    }
-
-    auto ctx = create_context_callback<
-      ImageReplayer, &ImageReplayer<I>::handle_bootstrap>(this);
-    request = BootstrapRequest<I>::create(
-      m_local_ioctx, m_remote_image.io_ctx, m_instance_watcher,
-      &m_local_image_ctx, m_local_image_id, m_remote_image.image_id,
-      m_global_image_id, m_threads->work_queue, m_threads->timer,
-      &m_threads->timer_lock, m_local_mirror_uuid, m_remote_image.mirror_uuid,
-      m_remote_journaler, &m_client_state, &m_client_meta, ctx,
-      &m_resync_requested, &m_progress_cxt);
     request->get();
     m_bootstrap_request = request;
   }
@@ -538,7 +513,7 @@ void ImageReplayer<I>::bootstrap() {
 
 template <typename I>
 void ImageReplayer<I>::handle_bootstrap(int r) {
-  dout(20) << "r=" << r << dendl;
+  dout(10) << "r=" << r << dendl;
   {
     Mutex::Locker locker(m_lock);
     m_bootstrap_request->put();
@@ -548,7 +523,9 @@ void ImageReplayer<I>::handle_bootstrap(int r) {
     }
   }
 
-  if (r == -EREMOTEIO) {
+  if (on_start_interrupted()) {
+    return;
+  } else if (r == -EREMOTEIO) {
     m_local_image_tag_owner = "";
     dout(5) << "remote image is non-primary" << dendl;
     on_start_fail(-EREMOTEIO, "remote image is non-primary");
@@ -560,8 +537,6 @@ void ImageReplayer<I>::handle_bootstrap(int r) {
   } else if (r < 0) {
     on_start_fail(r, "error bootstrapping replay");
     return;
-  } else if (on_start_interrupted()) {
-    return;
   } else if (m_resync_requested) {
     on_start_fail(0, "resync requested");
     return;
@@ -587,7 +562,7 @@ void ImageReplayer<I>::handle_bootstrap(int r) {
 
 template <typename I>
 void ImageReplayer<I>::init_remote_journaler() {
-  dout(20) << dendl;
+  dout(10) << dendl;
 
   Context *ctx = create_context_callback<
     ImageReplayer, &ImageReplayer<I>::handle_init_remote_journaler>(this);
@@ -596,14 +571,14 @@ void ImageReplayer<I>::init_remote_journaler() {
 
 template <typename I>
 void ImageReplayer<I>::handle_init_remote_journaler(int r) {
-  dout(20) << "r=" << r << dendl;
+  dout(10) << "r=" << r << dendl;
 
-  if (r < 0) {
+  if (on_start_interrupted()) {
+    return;
+  } else if (r < 0) {
     derr << "failed to initialize remote journal: " << cpp_strerror(r) << dendl;
     on_start_fail(r, "error initializing remote journal");
     return;
-  } else if (on_start_interrupted()) {
-    return;
   }
 
   m_remote_journaler->add_listener(&m_remote_listener);
@@ -637,7 +612,7 @@ void ImageReplayer<I>::handle_init_remote_journaler(int r) {
 
 template <typename I>
 void ImageReplayer<I>::start_replay() {
-  dout(20) << dendl;
+  dout(10) << dendl;
 
   Context *start_ctx = create_context_callback<
     ImageReplayer, &ImageReplayer<I>::handle_start_replay>(this);
@@ -646,7 +621,7 @@ void ImageReplayer<I>::start_replay() {
 
 template <typename I>
 void ImageReplayer<I>::handle_start_replay(int r) {
-  dout(20) << "r=" << r << dendl;
+  dout(10) << "r=" << r << dendl;
 
   if (r < 0) {
     assert(m_local_replay == nullptr);
@@ -687,12 +662,12 @@ void ImageReplayer<I>::handle_start_replay(int r) {
     m_replay_handler = new ReplayHandler<I>(this);
     m_remote_journaler->start_live_replay(m_replay_handler, poll_seconds);
 
-    dout(20) << "m_remote_journaler=" << *m_remote_journaler << dendl;
+    dout(10) << "m_remote_journaler=" << *m_remote_journaler << dendl;
   }
 
-  dout(10) << "start succeeded" << dendl;
+  dout(20) << "start succeeded" << dendl;
   if (on_finish != nullptr) {
-    dout(10) << "on finish complete, r=" << r << dendl;
+    dout(20) << "on finish complete, r=" << r << dendl;
     on_finish->complete(r);
   }
 }
@@ -700,7 +675,7 @@ void ImageReplayer<I>::handle_start_replay(int r) {
 template <typename I>
 void ImageReplayer<I>::on_start_fail(int r, const std::string &desc)
 {
-  dout(10) << "r=" << r << dendl;
+  dout(20) << "r=" << r << dendl;
   Context *ctx = new FunctionContext([this, r, desc](int _r) {
       {
         Mutex::Locker locker(m_lock);
@@ -709,12 +684,14 @@ void ImageReplayer<I>::on_start_fail(int r, const std::string &desc)
         if (r < 0 && r != -ECANCELED && r != -EREMOTEIO && r != -ENOENT) {
           derr << "start failed: " << cpp_strerror(r) << dendl;
         } else {
-          dout(10) << "start canceled" << dendl;
+          dout(20) << "start canceled" << dendl;
         }
       }
 
       set_state_description(r, desc);
-      update_mirror_image_status(false, boost::none);
+      if (m_local_ioctx) {
+        update_mirror_image_status(false, boost::none);
+      }
       reschedule_update_status_task(-1);
       shut_down(r);
     });
@@ -722,20 +699,15 @@ void ImageReplayer<I>::on_start_fail(int r, const std::string &desc)
 }
 
 template <typename I>
-bool ImageReplayer<I>::on_start_interrupted() {
+bool ImageReplayer<I>::on_start_interrupted()
+{
   Mutex::Locker locker(m_lock);
-  return on_start_interrupted(m_lock);
-}
-
-template <typename I>
-bool ImageReplayer<I>::on_start_interrupted(Mutex& lock) {
-  assert(m_lock.is_locked());
   assert(m_state == STATE_STARTING);
-  if (!m_stop_requested) {
+  if (m_on_stop_finish == nullptr) {
     return false;
   }
 
-  on_start_fail(-ECANCELED);
+  on_start_fail(-ECANCELED, "");
   return true;
 }
 
@@ -743,11 +715,9 @@ template <typename I>
 void ImageReplayer<I>::stop(Context *on_finish, bool manual, int r,
 			    const std::string& desc)
 {
-  dout(10) << "on_finish=" << on_finish << ", manual=" << manual
+  dout(20) << "on_finish=" << on_finish << ", manual=" << manual
 	   << ", desc=" << desc << dendl;
 
-  m_image_deleter->cancel_waiter(m_local_pool_id, m_global_image_id);
-
   image_replayer::BootstrapRequest<I> *bootstrap_request = nullptr;
   bool shut_down_replay = false;
   bool running = true;
@@ -759,13 +729,13 @@ void ImageReplayer<I>::stop(Context *on_finish, bool manual, int r,
     } else {
       if (!is_stopped_()) {
 	if (m_state == STATE_STARTING) {
-	  dout(10) << "canceling start" << dendl;
-	  if (m_bootstrap_request != nullptr) {
+	  dout(20) << "canceling start" << dendl;
+	  if (m_bootstrap_request) {
             bootstrap_request = m_bootstrap_request;
             bootstrap_request->get();
 	  }
 	} else {
-	  dout(10) << "interrupting replay" << dendl;
+	  dout(20) << "interrupting replay" << dendl;
 	  shut_down_replay = true;
 	}
 
@@ -779,7 +749,6 @@ void ImageReplayer<I>::stop(Context *on_finish, bool manual, int r,
 
   // avoid holding lock since bootstrap request will update status
   if (bootstrap_request != nullptr) {
-    dout(10) << "canceling bootstrap" << dendl;
     bootstrap_request->cancel();
     bootstrap_request->put();
   }
@@ -802,7 +771,7 @@ void ImageReplayer<I>::stop(Context *on_finish, bool manual, int r,
 template <typename I>
 void ImageReplayer<I>::on_stop_journal_replay(int r, const std::string &desc)
 {
-  dout(10) << dendl;
+  dout(20) << "enter" << dendl;
 
   {
     Mutex::Locker locker(m_lock);
@@ -823,7 +792,7 @@ void ImageReplayer<I>::on_stop_journal_replay(int r, const std::string &desc)
 template <typename I>
 void ImageReplayer<I>::handle_replay_ready()
 {
-  dout(20) << dendl;
+  dout(20) << "enter" << dendl;
   if (on_replay_interrupted()) {
     return;
   }
@@ -868,7 +837,7 @@ void ImageReplayer<I>::restart(Context *on_finish)
 template <typename I>
 void ImageReplayer<I>::flush(Context *on_finish)
 {
-  dout(10) << dendl;
+  dout(20) << "enter" << dendl;
 
   {
     Mutex::Locker locker(m_lock);
@@ -892,7 +861,7 @@ void ImageReplayer<I>::flush(Context *on_finish)
 template <typename I>
 void ImageReplayer<I>::on_flush_local_replay_flush_start(Context *on_flush)
 {
-  dout(10) << dendl;
+  dout(20) << "enter" << dendl;
   FunctionContext *ctx = new FunctionContext(
     [this, on_flush](int r) {
       on_flush_local_replay_flush_finish(on_flush, r);
@@ -907,7 +876,7 @@ template <typename I>
 void ImageReplayer<I>::on_flush_local_replay_flush_finish(Context *on_flush,
                                                           int r)
 {
-  dout(10) << "r=" << r << dendl;
+  dout(20) << "r=" << r << dendl;
   if (r < 0) {
     derr << "error flushing local replay: " << cpp_strerror(r) << dendl;
     on_flush->complete(r);
@@ -961,7 +930,7 @@ bool ImageReplayer<I>::on_replay_interrupted()
 template <typename I>
 void ImageReplayer<I>::print_status(Formatter *f, stringstream *ss)
 {
-  dout(10) << dendl;
+  dout(20) << "enter" << dendl;
 
   Mutex::Locker l(m_lock);
 
@@ -979,27 +948,28 @@ void ImageReplayer<I>::print_status(Formatter *f, stringstream *ss)
 template <typename I>
 void ImageReplayer<I>::handle_replay_complete(int r, const std::string &error_desc)
 {
-  dout(10) << "r=" << r << dendl;
+  dout(20) << "r=" << r << dendl;
   if (r < 0) {
     derr << "replay encountered an error: " << cpp_strerror(r) << dendl;
+    set_state_description(r, error_desc);
   }
 
   {
     Mutex::Locker locker(m_lock);
     m_stop_requested = true;
   }
-  on_stop_journal_replay(r, error_desc);
+  on_replay_interrupted();
 }
 
 template <typename I>
 void ImageReplayer<I>::replay_flush() {
-  dout(10) << dendl;
+  dout(20) << dendl;
 
   bool interrupted = false;
   {
     Mutex::Locker locker(m_lock);
     if (m_state != STATE_REPLAYING) {
-      dout(10) << "replay interrupted" << dendl;
+      dout(20) << "replay interrupted" << dendl;
       interrupted = true;
     } else {
       m_state = STATE_REPLAY_FLUSHING;
@@ -1031,7 +1001,7 @@ void ImageReplayer<I>::replay_flush() {
 
 template <typename I>
 void ImageReplayer<I>::handle_replay_flush(int r) {
-  dout(10) << "r=" << r << dendl;
+  dout(20) << "r=" << r << dendl;
 
   {
     Mutex::Locker locker(m_lock);
@@ -1054,7 +1024,7 @@ void ImageReplayer<I>::handle_replay_flush(int r) {
 
 template <typename I>
 void ImageReplayer<I>::get_remote_tag() {
-  dout(20) << "tag_tid: " << m_replay_tag_tid << dendl;
+  dout(15) << "tag_tid: " << m_replay_tag_tid << dendl;
 
   Context *ctx = create_context_callback<
     ImageReplayer, &ImageReplayer<I>::handle_get_remote_tag>(this);
@@ -1063,12 +1033,12 @@ void ImageReplayer<I>::get_remote_tag() {
 
 template <typename I>
 void ImageReplayer<I>::handle_get_remote_tag(int r) {
-  dout(20) << "r=" << r << dendl;
+  dout(15) << "r=" << r << dendl;
 
   if (r == 0) {
     try {
       bufferlist::iterator it = m_replay_tag.data.begin();
-      ::decode(m_replay_tag_data, it);
+      decode(m_replay_tag_data, it);
     } catch (const buffer::error &err) {
       r = -EBADMSG;
     }
@@ -1083,7 +1053,7 @@ void ImageReplayer<I>::handle_get_remote_tag(int r) {
   }
 
   m_replay_tag_valid = true;
-  dout(20) << "decoded remote tag " << m_replay_tag_tid << ": "
+  dout(15) << "decoded remote tag " << m_replay_tag_tid << ": "
            << m_replay_tag_data << dendl;
 
   allocate_local_tag();
@@ -1277,7 +1247,7 @@ void ImageReplayer<I>::handle_process_entry_safe(const ReplayEntry& replay_entry
 template <typename I>
 bool ImageReplayer<I>::update_mirror_image_status(bool force,
                                                   const OptionalState &state) {
-  dout(15) << dendl;
+  dout(20) << dendl;
   {
     Mutex::Locker locker(m_lock);
     if (!start_mirror_image_status_update(force, false)) {
@@ -1296,17 +1266,17 @@ bool ImageReplayer<I>::start_mirror_image_status_update(bool force,
 
   if (!force && !is_stopped_()) {
     if (!is_running_()) {
-      dout(15) << "shut down in-progress: ignoring update" << dendl;
+      dout(20) << "shut down in-progress: ignoring update" << dendl;
       return false;
     } else if (m_in_flight_status_updates > (restarting ? 1 : 0)) {
-      dout(15) << "already sending update" << dendl;
+      dout(20) << "already sending update" << dendl;
       m_update_status_requested = true;
       return false;
     }
   }
 
-  dout(20) << dendl;
   ++m_in_flight_status_updates;
+  dout(15) << "in-flight updates=" << m_in_flight_status_updates << dendl;
   return true;
 }
 
@@ -1319,7 +1289,7 @@ void ImageReplayer<I>::finish_mirror_image_status_update() {
     Mutex::Locker locker(m_lock);
     assert(m_in_flight_status_updates > 0);
     if (--m_in_flight_status_updates > 0) {
-      dout(20) << "waiting on " << m_in_flight_status_updates << " in-flight "
+      dout(15) << "waiting on " << m_in_flight_status_updates << " in-flight "
                << "updates" << dendl;
       return;
     }
@@ -1327,7 +1297,7 @@ void ImageReplayer<I>::finish_mirror_image_status_update() {
     std::swap(on_finish, m_on_update_status_finish);
   }
 
-  dout(20) << dendl;
+  dout(15) << dendl;
   if (on_finish != nullptr) {
     on_finish->complete(0);
   }
@@ -1335,7 +1305,7 @@ void ImageReplayer<I>::finish_mirror_image_status_update() {
 
 template <typename I>
 void ImageReplayer<I>::queue_mirror_image_status_update(const OptionalState &state) {
-  dout(15) << dendl;
+  dout(20) << dendl;
   FunctionContext *ctx = new FunctionContext(
     [this, state](int r) {
       send_mirror_status_update(state);
@@ -1350,8 +1320,8 @@ void ImageReplayer<I>::send_mirror_status_update(const OptionalState &opt_state)
   int last_r;
   bool stopping_replay;
 
-  OptionalMirrorImageStatusState mirror_image_status_state{
-    boost::make_optional(false, cls::rbd::MirrorImageStatusState{})};
+  OptionalMirrorImageStatusState mirror_image_status_state =
+    boost::make_optional(false, cls::rbd::MIRROR_IMAGE_STATUS_STATE_UNKNOWN);
   image_replayer::BootstrapRequest<I>* bootstrap_request = nullptr;
   {
     Mutex::Locker locker(m_lock);
@@ -1397,7 +1367,7 @@ void ImageReplayer<I>::send_mirror_status_update(const OptionalState &opt_state)
     {
       Context *on_req_finish = new FunctionContext(
         [this](int r) {
-          dout(15) << "replay status ready: r=" << r << dendl;
+          dout(20) << "replay status ready: r=" << r << dendl;
           if (r >= 0) {
             send_mirror_status_update(boost::none);
           } else if (r == -EAGAIN) {
@@ -1407,13 +1377,15 @@ void ImageReplayer<I>::send_mirror_status_update(const OptionalState &opt_state)
         });
 
       std::string desc;
+      assert(m_replay_status_formatter != nullptr);
       if (!m_replay_status_formatter->get_or_send_update(&desc,
                                                          on_req_finish)) {
-        dout(15) << "waiting for replay status" << dendl;
+        dout(20) << "waiting for replay status" << dendl;
         return;
       }
       status.description = "replaying, " + desc;
-      mirror_image_status_state = boost::none;
+      mirror_image_status_state = boost::make_optional(
+        false, cls::rbd::MIRROR_IMAGE_STATUS_STATE_UNKNOWN);
     }
     break;
   case STATE_STOPPING:
@@ -1453,20 +1425,21 @@ void ImageReplayer<I>::send_mirror_status_update(const OptionalState &opt_state)
     status.state = *mirror_image_status_state;
   }
 
-  dout(20) << "status=" << status << dendl;
+  dout(15) << "status=" << status << dendl;
   librados::ObjectWriteOperation op;
   librbd::cls_client::mirror_image_status_set(&op, m_global_image_id, status);
 
+  assert(m_local_ioctx);
   librados::AioCompletion *aio_comp = create_rados_callback<
     ImageReplayer<I>, &ImageReplayer<I>::handle_mirror_status_update>(this);
-  int r = m_local_ioctx.aio_operate(RBD_MIRRORING, aio_comp, &op);
+  int r = m_local_ioctx->aio_operate(RBD_MIRRORING, aio_comp, &op);
   assert(r == 0);
   aio_comp->release();
 }
 
 template <typename I>
 void ImageReplayer<I>::handle_mirror_status_update(int r) {
-  dout(20) << "r=" << r << dendl;
+  dout(15) << "r=" << r << dendl;
 
   bool running = false;
   bool started = false;
@@ -1486,7 +1459,7 @@ void ImageReplayer<I>::handle_mirror_status_update(int r) {
   if (started) {
     queue_mirror_image_status_update(boost::none);
   } else if (running) {
-    reschedule_update_status_task();
+    reschedule_update_status_task(0);
   }
 
   // mark committed status update as no longer in-flight
@@ -1495,14 +1468,14 @@ void ImageReplayer<I>::handle_mirror_status_update(int r) {
 
 template <typename I>
 void ImageReplayer<I>::reschedule_update_status_task(int new_interval) {
-  dout(20) << dendl;
-
   bool canceled_task = false;
   {
     Mutex::Locker locker(m_lock);
     Mutex::Locker timer_locker(m_threads->timer_lock);
 
     if (m_update_status_task) {
+      dout(15) << "canceling existing status update task" << dendl;
+
       canceled_task = m_threads->timer->cancel_event(m_update_status_task);
       m_update_status_task = nullptr;
     }
@@ -1520,13 +1493,14 @@ void ImageReplayer<I>::reschedule_update_status_task(int new_interval) {
 
           queue_mirror_image_status_update(boost::none);
         });
+      dout(15) << "scheduling status update task after "
+               << m_update_status_interval << " seconds" << dendl;
       m_threads->timer->add_event_after(m_update_status_interval,
                                         m_update_status_task);
     }
   }
 
   if (canceled_task) {
-    dout(20) << "canceled task" << dendl;
     // decrement in-flight status update counter for canceled task
     finish_mirror_image_status_update();
   }
@@ -1534,7 +1508,7 @@ void ImageReplayer<I>::reschedule_update_status_task(int new_interval) {
 
 template <typename I>
 void ImageReplayer<I>::shut_down(int r) {
-  dout(20) << "r=" << r << dendl;
+  dout(10) << "r=" << r << dendl;
 
   bool canceled_delayed_preprocess_task = false;
   {
@@ -1551,6 +1525,8 @@ void ImageReplayer<I>::shut_down(int r) {
     m_event_replay_tracker.finish_op();
   }
 
+  reschedule_update_status_task(-1);
+
   {
     Mutex::Locker locker(m_lock);
     assert(m_state == STATE_STOPPING);
@@ -1559,7 +1535,7 @@ void ImageReplayer<I>::shut_down(int r) {
     // before proceeding
     if (m_in_flight_status_updates > 0) {
       if (m_on_update_status_finish == nullptr) {
-        dout(15) << "waiting for in-flight status update" << dendl;
+        dout(20) << "waiting for in-flight status update" << dendl;
         m_on_update_status_finish = new FunctionContext(
           [this, r](int _r) {
             shut_down(r);
@@ -1576,7 +1552,9 @@ void ImageReplayer<I>::shut_down(int r) {
   // chain the shut down sequence (reverse order)
   Context *ctx = new FunctionContext(
     [this, r](int _r) {
-      update_mirror_image_status(true, STATE_STOPPED);
+      if (m_local_ioctx) {
+        update_mirror_image_status(true, STATE_STOPPED);
+      }
       handle_shut_down(r);
     });
 
@@ -1662,6 +1640,8 @@ template <typename I>
 void ImageReplayer<I>::handle_shut_down(int r) {
   reschedule_update_status_task(-1);
 
+  bool resync_requested = false;
+  bool delete_requested = false;
   bool unregister_asok_hook = false;
   {
     Mutex::Locker locker(m_lock);
@@ -1670,7 +1650,7 @@ void ImageReplayer<I>::handle_shut_down(int r) {
     // before proceeding
     if (m_in_flight_status_updates > 0) {
       if (m_on_update_status_finish == nullptr) {
-        dout(15) << "waiting for in-flight status update" << dendl;
+        dout(20) << "waiting for in-flight status update" << dendl;
         m_on_update_status_finish = new FunctionContext(
           [this, r](int _r) {
             handle_shut_down(r);
@@ -1679,24 +1659,16 @@ void ImageReplayer<I>::handle_shut_down(int r) {
       return;
     }
 
-    bool delete_requested = false;
     if (m_delete_requested && !m_local_image_id.empty()) {
       assert(m_remote_image.image_id.empty());
       dout(0) << "remote image no longer exists: scheduling deletion" << dendl;
-      delete_requested = true;
+      unregister_asok_hook = true;
+      std::swap(delete_requested, m_delete_requested);
     }
-    if (delete_requested || m_resync_requested) {
-      m_image_deleter->schedule_image_delete(m_local,
-                                             m_local_pool_id,
-                                             m_global_image_id,
-                                             m_resync_requested);
 
+    std::swap(resync_requested, m_resync_requested);
+    if (delete_requested || resync_requested) {
       m_local_image_id = "";
-      m_resync_requested = false;
-      if (m_delete_requested) {
-        unregister_asok_hook = true;
-        m_delete_requested = false;
-      }
     } else if (m_last_r == -ENOENT &&
                m_local_image_id.empty() && m_remote_image.image_id.empty()) {
       dout(0) << "mirror image no longer exists" << dendl;
@@ -1709,9 +1681,17 @@ void ImageReplayer<I>::handle_shut_down(int r) {
     unregister_admin_socket_hook();
   }
 
-  dout(20) << "stop complete" << dendl;
-  m_local_ioctx.close();
+  if (delete_requested || resync_requested) {
+    dout(5) << "moving image to trash" << dendl;
+    auto ctx = new FunctionContext([this, r](int) {
+      handle_shut_down(r);
+    });
+    ImageDeleter<I>::trash_move(*m_local_ioctx, m_global_image_id,
+                                resync_requested, m_threads->work_queue, ctx);
+    return;
+  }
 
+  dout(10) << "stop complete" << dendl;
   ReplayStatusFormatter<I>::destroy(m_replay_status_formatter);
   m_replay_status_formatter = nullptr;
 
@@ -1728,12 +1708,12 @@ void ImageReplayer<I>::handle_shut_down(int r) {
   }
 
   if (on_start != nullptr) {
-    dout(10) << "on start finish complete, r=" << r << dendl;
+    dout(20) << "on start finish complete, r=" << r << dendl;
     on_start->complete(r);
     r = 0;
   }
   if (on_stop != nullptr) {
-    dout(10) << "on stop finish complete, r=" << r << dendl;
+    dout(20) << "on stop finish complete, r=" << r << dendl;
     on_stop->complete(r);
   }
 }
@@ -1783,7 +1763,7 @@ std::string ImageReplayer<I>::to_string(const State state) {
 
 template <typename I>
 void ImageReplayer<I>::resync_image(Context *on_finish) {
-  dout(10) << dendl;
+  dout(20) << dendl;
 
   m_resync_requested = true;
   stop(on_finish);
@@ -1827,8 +1807,8 @@ template <typename I>
 void ImageReplayer<I>::reregister_admin_socket_hook() {
   {
     Mutex::Locker locker(m_lock);
-    auto name = m_local_ioctx.get_pool_name() + "/" + m_local_image_name;
-    if (m_asok_hook != nullptr && m_name == name) {
+    auto name = m_local_ioctx->get_pool_name() + "/" + m_local_image_name;
+    if (m_name == name) {
       return;
     }
     m_name = name;
