@@ -1,8 +1,6 @@
 // -*- mode:C++; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
 // vim: ts=8 sw=2 smarttab
 
-#include <signal.h>
-
 #include <boost/range/adaptor/map.hpp>
 
 #include "common/Formatter.h"
@@ -180,8 +178,8 @@ public:
     }
   }
 
-  bool call(std::string command, cmdmap_t& cmdmap, std::string format,
-	    bufferlist& out) override {
+  bool call(std::string_view command, const cmdmap_t& cmdmap,
+	    std::string_view format, bufferlist& out) override {
     Commands::const_iterator i = commands.find(command);
     assert(i != commands.end());
     Formatter *f = Formatter::create(format);
@@ -193,7 +191,7 @@ public:
   }
 
 private:
-  typedef std::map<std::string, MirrorAdminSocketCommand*> Commands;
+  typedef std::map<std::string, MirrorAdminSocketCommand*, std::less<>> Commands;
 
   AdminSocket *admin_socket;
   Commands commands;
@@ -206,8 +204,9 @@ Mirror::Mirror(CephContext *cct, const std::vector<const char*> &args) :
   m_local(new librados::Rados()),
   m_asok_hook(new MirrorAdminSocketHook(cct, this))
 {
-  cct->lookup_or_create_singleton_object<Threads<librbd::ImageCtx> >(
-    m_threads, "rbd_mirror::threads");
+  m_threads =
+    &(cct->lookup_or_create_singleton_object<Threads<librbd::ImageCtx>>(
+	"rbd_mirror::threads", false, cct));
   m_service_daemon.reset(new ServiceDaemon<>(m_cct, m_local, m_threads));
 }
 
@@ -218,26 +217,10 @@ Mirror::~Mirror()
 
 void Mirror::handle_signal(int signum)
 {
-  dout(20) << signum << dendl;
-
-  Mutex::Locker l(m_lock);
-
-  switch (signum) {
-  case SIGHUP:
-    for (auto &it : m_pool_replayers) {
-      it.second->reopen_logs();
-    }
-    g_ceph_context->reopen_logs();
-    break;
-
-  case SIGINT:
-  case SIGTERM:
-    m_stopping = true;
+  m_stopping = true;
+  {
+    Mutex::Locker l(m_lock);
     m_cond.Signal();
-    break;
-
-  default:
-    ceph_abort();
   }
 }
 
@@ -263,11 +246,6 @@ int Mirror::init()
 
   m_local_cluster_watcher.reset(new ClusterWatcher(m_local, m_lock,
                                                    m_service_daemon.get()));
-
-  m_image_deleter.reset(new ImageDeleter<>(m_threads->work_queue,
-                                           m_threads->timer,
-                                           &m_threads->timer_lock,
-                                           m_service_daemon.get()));
   return r;
 }
 
@@ -314,10 +292,7 @@ void Mirror::print_status(Formatter *f, stringstream *ss)
 
   if (f) {
     f->close_section();
-    f->open_object_section("image_deleter");
   }
-
-  m_image_deleter->print_status(f, ss);
 }
 
 void Mirror::start()
@@ -436,9 +411,8 @@ void Mirror::update_pool_replayers(const PoolPeers &pool_peers)
         }
       } else {
         dout(20) << "starting pool replayer for " << peer << dendl;
-        unique_ptr<PoolReplayer> pool_replayer(new PoolReplayer(
-	  m_threads, m_service_daemon.get(), m_image_deleter.get(), kv.first,
-          peer, m_args));
+        unique_ptr<PoolReplayer<>> pool_replayer(new PoolReplayer<>(
+	  m_threads, m_service_daemon.get(), kv.first, peer, m_args));
 
         // TODO: make async
         pool_replayer->init();
